region_names_dict = {
        0: 'Unknown',
        1: 'Left-Lateral-Ventricle',
 2: 'Left-Inf-Lat-Vent',
 3: 'Left-Cerebellum-White-Matter',
 4: 'Left-Cerebellum-Cortex',
 5: 'Left-Thalamus-Proper',
 6: 'Left-Caudate',
 7: 'Left-Putamen',
 8: 'Left-Pallidum',
 9: '3rd-Ventricle',
 10: '4th-Ventricle',
 11: 'Brain-Stem',
 12: 'Left-Hippocampus',
 13: 'Left-Amygdala',
 14: 'CSF',
 15: 'Left-Accumbens-area',
 16: 'Left-VentralDC',
 17: 'Left-vessel',
 18: 'Left-choroid-plexus',
 19: 'Right-Cerebral-White-Matter',
 20: 'Right-Lateral-Ventricle',
 21: 'Right-Inf-Lat-Vent',
 22: 'Right-Cerebellum-White-Matter',
 23: 'Right-Cerebellum-Cortex',
 24: 'Right-Thalamus-Proper',
 25: 'Right-Caudate',
 26: 'Right-Putamen',
 27: 'Right-Pallidum',
 28: 'Right-Hippocampus',
 29: 'Right-Amygdala',
 30: 'Right-Accumbens-area',
 31: 'Right-VentralDC',
 32: 'Right-vessel',
 33: 'Right-choroid-plexus',
 34: 'Optic-Chiasm',
 35: 'CC_Posterior',
 36: 'CC_Mid_Posterior',
 37: 'CC_Central',
 38: 'CC_Mid_Anterior',
 39: 'CC_Anterior',
 40: 'ctx-lh-caudalanteriorcingulate',
 41: 'ctx-lh-caudalmiddlefrontal',
 42: 'ctx-lh-cuneus',
 43: 'ctx-lh-entorhinal',
 44: 'ctx-lh-fusiform',
 45: 'ctx-lh-inferiorparietal',
 46: 'ctx-lh-inferiortemporal',
 47: 'ctx-lh-isthmuscingulate',
 48: 'ctx-lh-lateraloccipital',
 49: 'ctx-lh-lateralorbitofrontal',
 50: 'ctx-lh-lingual',
 51: 'ctx-lh-medialorbitofrontal',
 52: 'ctx-lh-middletemporal',
 53: 'ctx-lh-parahippocampal',
 54: 'ctx-lh-paracentral',
 55: 'ctx-lh-parsopercularis',
 56: 'ctx-lh-parsorbitalis',
 57: 'ctx-lh-parstriangularis',
 58: 'ctx-lh-pericalcarine',
 59: 'ctx-lh-postcentral',
 60: 'ctx-lh-posteriorcingulate',
 61: 'ctx-lh-precentral',
 62: 'ctx-lh-precuneus',
 63: 'ctx-lh-rostralanteriorcingulate',
 64: 'ctx-lh-rostralmiddlefrontal',
 65: 'ctx-lh-superiorfrontal',
 66: 'ctx-lh-superiorparietal',
 67: 'ctx-lh-superiortemporal',
 68: 'ctx-lh-supramarginal',
 69: 'ctx-lh-transversetemporal',
 70: 'ctx-lh-insula',
 71: 'ctx-rh-caudalanteriorcingulate',
 72: 'ctx-rh-caudalmiddlefrontal',
 73: 'ctx-rh-cuneus',
 74: 'ctx-rh-entorhinal',
 75: 'ctx-rh-fusiform',
 76: 'ctx-rh-inferiorparietal',
 77: 'ctx-rh-inferiortemporal',
 78: 'ctx-rh-isthmuscingulate',
 79: 'ctx-rh-lateraloccipital',
 80: 'ctx-rh-lateralorbitofrontal',
 81: 'ctx-rh-lingual',
 82: 'ctx-rh-medialorbitofrontal',
 83: 'ctx-rh-middletemporal',
 84: 'ctx-rh-parahippocampal',
 85: 'ctx-rh-paracentral',
 86: 'ctx-rh-parsopercularis',
 87: 'ctx-rh-parsorbitalis',
 88: 'ctx-rh-parstriangularis',
 89: 'ctx-rh-pericalcarine',
 90: 'ctx-rh-postcentral',
 91: 'ctx-rh-posteriorcingulate',
 92: 'ctx-rh-precentral',
 93: 'ctx-rh-precuneus',
 94: 'ctx-rh-rostralanteriorcingulate',
 95: 'ctx-rh-rostralmiddlefrontal',
 96: 'ctx-rh-superiorfrontal',
 97: 'ctx-rh-superiorparietal',
 98: 'ctx-rh-superiortemporal',
 99: 'ctx-rh-supramarginal',
 100: 'ctx-rh-transversetemporal',
 101: 'ctx-rh-insula',
 102: 'Left-Cerebral-White-Matter'
      }
